@@ -8,12 +8,40 @@
 
 
 /*
- * Copyright (c) 2001 Qualcomm Incorporated.  All rights reserved.
+ * Copyright (c) 2006 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  *
  * Revisions:
  *
+ *     04/11/05 [RCG]
+ *              - Add 'x' command-line option to disable XTND XMIT.
+ *
+ *     12/15/04 [RCG]
+ *              - Apply patch from Kenny Nguyen to fix core dump
+ *                on 64-bit Solaris 2.8.
+ *
+ *     12/14/04 [RCG]
+ *              - EXPIRE now defaults to NEVER
+ *
+ *     12/04/04 [RCG]
+ *              - Apply patch from David Shrimpton to to make
+ *                "Servicing request" logged always and not just
+ *                when debugging is on.   This allows start of pop
+ *                sessions to be logged always which is useful for
+ *                diagnosis of problems.
+ *
+ *     02/06/03 [RCG]
+ *              - Apply patch from Jeremy Chadwick to fix pathname
+ *                trimming of arg[0].
+ *
+ *     11/16/01 [secabeen]
+ *              - Added home-dir-misc to put .cache and .pop files in 
+ *                user home directories
+ *
+ *     07/25/01 [RCG]
+ *              - Allow '-p' to be used even if APOP not defined.
+ *
  *     02/12/01 [RCG]
  *              - Save trace file name.
  *
@@ -93,6 +121,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
+#include <limits.h>
 
 #if HAVE_SYS_NETINET_IN_H
 #  include <sys/netinet/in.h>
@@ -463,11 +492,19 @@ char    **      argmessage;
      * Note that we zeroed the block above, so we really only need to
      * set any desired non-zero/non-NULL values.
      */
-    p->bulldir             = BULLDIR;
-    p->AuthType            = noauth;
-    p->AuthState           = none;
-    p->AllowClearText      = ClearTextDefault;
-    p->check_lock_refresh  = 5000; /* just a wild guess */
+    p->bulldir                  = BULLDIR;
+    p->AuthType                 = noauth;
+    p->AuthState                = none;
+    p->AllowClearText           = ClearTextDefault;
+    p->check_lock_refresh       = 5000; /* just a wild guess */
+    p->expire                   = -1;   /* default to EXPIRE NEVER */
+    p->bXtnd_Xmit               = TRUE; /* XTND XMIT enabled by default */
+
+#ifdef    CYRUS_SASL
+    p->sasl.sasl_no_plaintext   = TRUE;
+    p->sasl.sasl_no_anonymous   = TRUE;
+    p->sasl.sasl_max_ssf        = UINT_MAX;
+#endif /* CYRUS_SASL */
 
 #ifdef     KERBEROS
     p->kerberos_service    = KERBEROS_SERVICE;
@@ -553,6 +590,11 @@ char    **      argmessage;
     DEBUG_LOG1 ( p, "Spool is %s in user's home dir", p->pHome_dir_mail );
 #endif /* HOMEDIRMAIL */
 
+#ifdef    HOMEDIRMISC
+    p->bHome_dir_misc      = TRUE;
+    DEBUG_LOG0 ( p, ".pop and .cache files are stored in user's home dir");
+#endif /* HOMEDIRMISC*/
+
 #ifdef    OLD_STYLE_UIDL
     p->bOld_style_uid      = TRUE;
     DEBUG_LOG0 ( p, "Generating old (pre-3.x) style UIDs" );
@@ -663,7 +705,15 @@ char    **      argmessage;
     /*  
      * Save my name in a global variable 
      */
-    p->myname         = argmessage [ 0 ];
+    p->myname = argmessage [ 0 ];
+    if ( p->myname != NULL && *p->myname != '\0' )
+    {
+        char *ptr = NULL;
+        ptr = strrchr ( p->myname, '/' );
+        if ( ptr != NULL && strlen(ptr) > 1 )
+            p->myname = ptr + 1;
+    }
+
 
     /*  
      * Open the log file 
@@ -677,7 +727,9 @@ char    **      argmessage;
     /*  
      * Process command line arguments 
      */
-    while ( ( c = getopt ( argcount, argmessage, "b:BcCdD:e:f:FkK:l:L:p:RsSt:T:uUvy:") ) != EOF )
+    while ( ( c = getopt ( argcount, argmessage, 
+                           "b:BcCdD:e:f:FkK:l:L:p:RsSt:T:uUvVxy:" ) 
+            ) != EOF )
         switch ( c ) {
 
         case 'b': /* Bulletins requested */
@@ -788,7 +840,6 @@ char    **      argmessage;
                          p->check_lock_refresh );
             break;
 
-#if defined(APOP) || defined(SCRAM)
         case 'p': /*  Set clear text password handling */
             pw_handling = atoi ( optarg );
             switch ( pw_handling ) {
@@ -804,16 +855,35 @@ char    **      argmessage;
                 case 3:
                     p->AllowClearText = ClearTextLocal;
                     break;
+                case 4:
+                    p->AllowClearText = ClearTextTLS;
+                    break;
                 default:
                     fprintf ( stderr, "Unrecognized -p value; 0 = default; "
                                       "1 = never; 2 = always (fallback); "
-                                      "3 = local only\n" );
+                                      "3 = local only; 4 = TLS/SSL only\n" );
                     errflag++;
             }
             DEBUG_LOG1 ( p, "Plain-text password handling=%d (-p)",
                          pw_handling );
+
+#if !defined(APOP) && !defined(SCRAM)
+            if ( p->AllowClearText == ClearTextNever ) {
+                fprintf ( stderr, "Can't set -p to 1 without enabling APOP\n" );
+                errflag++;
+                DEBUG_LOG0 ( p, "Can't set -p to 1 without enabling APOP" );
+            }
+#endif /* not APOP and not SCRAM */
+
+#ifndef   QPOP_SSL
+            if ( p->AllowClearText == ClearTextTLS ) {
+                fprintf ( stderr, "Can't set -p to 4 without enabling TLS/SSL\n" );
+                errflag++;
+                DEBUG_LOG0 ( p, "Can't set -p to 1 without enabling TLS/SSL" );
+            }
+#endif /* QPOP_SSL */
+
             break;
-#endif /* APOP or SCRAM */
 
         case 'R': /*  Omit reverse lookups on client IP */
             no_rev_lookup = TRUE;
@@ -872,6 +942,12 @@ char    **      argmessage;
             break;
 
         case 'v': /* RESERVED (handled in main.c) -- DO NOT USE */
+        case 'V': /* RESERVED (handled in main.c) -- DO NOT USE */
+            break;
+
+        case 'x': /* prevent xtnd_xmit */
+            p->bXtnd_Xmit = FALSE;
+            DEBUG_LOG0 ( p, "Will not permit XTND XMIT (-x)" );
             break;
 
         case 'y': /* log facility */
@@ -886,6 +962,7 @@ char    **      argmessage;
                 fprintf ( stderr, "Error setting '-y' to %s\n", optarg );
                 errflag++;
             }
+            break;
 
         default: /*  Unknown option received */
             errflag++;
@@ -941,10 +1018,10 @@ char    **      argmessage;
                           "[-c] "
                           "[-C] "
                           "[-d] "
+                          "%s"          /* -D / drac-host, if DRAC set */
                           "[-e login_delay=nn,expire=nn] "
                           "[-f config-file] "
                           "[-F] "
-                          "%s"          /* -D / drac-host, if DRAC set */
                           "%s"          /* -k / kerberos, if KERBEROS set */
                           "%s"          /* -K / kerberos service, if KERBEROS set */
                           "%s"          /* -l / tls_support, if QPOP_SSL set */
@@ -958,6 +1035,8 @@ char    **      argmessage;
                           "[-u] "
                           "[-U] "
                           "[-v] "
+                          "[-V] "
+                          "[-y log-facility] "
                           "\n",
                   argmessage[0],
                   xB,                   /* -B (or not) */
@@ -1008,8 +1087,16 @@ char    **      argmessage;
              * you gave gethostbyname.  Thus do a reverse lookup
              * on the first address and hope for the best.
              */
-            u_long x = *(u_long *) hp->h_addr_list [ 0 ];
-            hp = gethostbyaddr ( (char *) &x, 4, AF_INET );
+
+            /*
+             * DO NOT use casting to a u_long.  This causes a Bus Error when
+             * running the 64-bit compiled executable on a Sun-Fire-280R SunOS 2.8 
+             * 64-bit.
+             */
+            struct in_addr sia;
+            sia = *(struct in_addr *) hp->h_addr_list[0];
+            hp = gethostbyaddr ( (char *) &sia, 4, AF_INET);
+
             if ( hp != NULL ) {
                 (void) strncpy ( p->myhost, hp->h_name, MAXHOSTNAMELEN );
                 p->myhost [ MAXHOSTNAMELEN ] = '\0';
@@ -1149,8 +1236,8 @@ char    **      argmessage;
         EXIT ( 1 );
     }
 
-    DEBUG_LOG3 ( p, "(v%s) Servicing request from \"%s\" at %s",
-                 VERSION, p->client, p->ipaddr );
+    pop_log ( p, POP_DEBUG, HERE, "(v%s) Servicing request from \"%s\" at %s",
+              VERSION, p->client, p->ipaddr );
     return ( kerb_authenticate ( p, &cs ) );
 }
 
