@@ -1,10 +1,19 @@
 /*
- * Copyright (c) 2001 QUALCOMM Incorporated. All rights reserved.
+ * Copyright (c) 2006 QUALCOMM Incorporated. All rights reserved.
  * See License.txt file for terms and conditions for modification and
  * redistribution.
  *
  * Revisions: 
  *
+ * 12/03/04  [rcg]
+ *         - Applied patch from Maks N. Polunin to (a) call SSL_shutdown()
+ *           again if it fails the first time, and to log when compiled
+ *           with debugging and debug is enabled (instead of either).
+ *
+ * 08/06/02  [rcg]
+ *         - Added ability to set options (SSL_OP_* values).
+ *         - Added debug trace call to show OpenSSL library version.
+ *
  * 05/13/01  [rcg]
  *         - Don't call SSL_shutdown unless we tried to negotiate an
  *           SSL session.  (As suggested by Kenneth Porter.)
@@ -54,6 +63,7 @@
 
 #ifdef    QPOP_OPENSSL
 #    include <openssl/ssl.h>
+#    include <openssl/opensslv.h>
 #    include <openssl/err.h>
 #    include <openssl/rand.h>
 #    include <openssl/crypto.h>
@@ -221,7 +231,8 @@ openssl_init ( pop_tls *pTLS, POP *pPOP )
     /*
      * First, initialize the SSL library
      */
-    DEBUG_LOG0 ( pPOP, "...Initializing OpenSSL library" );
+    DEBUG_LOG1 ( pPOP, "...Initializing OpenSSL library (version %s)",
+                 OPENSSL_VERSION_TEXT );
     SSL_load_error_strings();
     SSL_library_init(); /* always returns 1, no need to check */
 
@@ -340,16 +351,27 @@ openssl_init ( pop_tls *pTLS, POP *pPOP )
         goto Done;
     }
 
+    /*
+     * Set desired options
+     */
+    if ( pPOP->tls_options ) {
+        long opts = 0;
+        opts = SSL_CTX_set_options ( pTLS->m_OpenSSLctx, pPOP->tls_options );
+        DEBUG_LOG2 ( pPOP, "...set options %#0x; options now %#0lx",
+                     pPOP->tls_options, opts );
+    }
+
     /*
      * Establish the certificate for our server cert.
      */
     DEBUG_LOG1 ( pPOP, "...setting certificate file %s",
                  pPOP->tls_server_cert_file );
-    nErr = SSL_CTX_use_certificate_file ( pTLS->m_OpenSSLctx,
-                                          pPOP->tls_server_cert_file,
-                                          SSL_FILETYPE_PEM );
+    nErr = SSL_CTX_use_certificate_chain_file ( pTLS->m_OpenSSLctx,
+                                          pPOP->tls_server_cert_file );
+
     if ( nErr <= 0 ) {
-        log_openssl_err ( pPOP, HERE, "Error setting certificate PEM file %s",
+        log_openssl_err ( pPOP, HERE, 
+                          "Error setting certificate PEM file %s",
                           pPOP->tls_server_cert_file );
         goto Done;
     }
@@ -761,6 +783,16 @@ openssl_shutdown ( pop_tls *pTLS )
     if ( pTLS->m_pPOP->tls_started == TRUE ) {
         ret = SSL_shutdown ( pTLS->m_OpenSSLconn );
         DEBUG_LOG1 ( pTLS->m_pPOP, "tls shutdown returned %d", ret );
+        if ( ret == 0 ) {
+            /*
+             * 0 means the bidirectional shutdown is not yet complete;
+             * SSL_shutdown man page says to call SSL_shutdown() again
+             */
+            ret = SSL_shutdown ( pTLS->m_OpenSSLconn );
+            DEBUG_LOG1 ( pTLS->m_pPOP, 
+                         "... second tls shutdown returned %d", 
+                         ret );
+        }
     
         nErr = SSL_get_error ( pTLS->m_OpenSSLconn, ret );
         DEBUG_LOG2 ( pTLS->m_pPOP, "SSL_get_error says %s (%d)",
@@ -785,13 +817,13 @@ openssl_shutdown ( pop_tls *pTLS )
     
             case SSL_ERROR_SYSCALL:
                 rslt = -1;
-                if ( DEBUGGING || pTLS->m_pPOP->debug )
+                if ( DEBUGGING && pTLS->m_pPOP->debug )
                     log_openssl_err ( pTLS->m_pPOP, HERE, "TLS shutdown Error" );
                 break;
     
             case SSL_ERROR_SSL:
                 rslt = -1;
-                if ( DEBUGGING || pTLS->m_pPOP->debug )
+                if ( DEBUGGING && pTLS->m_pPOP->debug )
                     log_openssl_err ( pTLS->m_pPOP, HERE,
                                       "OpenSSL Error during shutdown" );
                 break;
